@@ -1,13 +1,12 @@
-# coding: utf-8
 """A tornado based Jupyter lab server."""
 
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
 
+import dataclasses
 import json
 import os
 import sys
-from os.path import join as pjoin
 
 from jupyter_core.application import JupyterApp, NoStart, base_aliases, base_flags
 from jupyter_server._version import version_info as jpserver_version_info
@@ -20,7 +19,7 @@ from jupyterlab_server import (
     WorkspaceImportApp,
     WorkspaceListApp,
 )
-from nbclassic.shim import NBClassicConfigShimMixin
+from notebook_shim.shim import NotebookConfigShimMixin
 from traitlets import Bool, Instance, Type, Unicode, default
 
 from ._version import __version__
@@ -43,6 +42,8 @@ from .commands import (
 )
 from .coreconfig import CoreConfig
 from .debuglog import DebugLogFileMixin
+from .extensions import MANAGERS as EXT_MANAGERS
+from .extensions.readonly import ReadOnlyExtensionManager
 from .handlers.announcements import (
     CheckForUpdate,
     CheckForUpdateABC,
@@ -53,11 +54,7 @@ from .handlers.announcements import (
 )
 from .handlers.build_handler import Builder, BuildHandler, build_path
 from .handlers.error_handler import ErrorHandler
-from .handlers.extension_manager_handler import (
-    ExtensionHandler,
-    ExtensionManager,
-    extensions_handler_path,
-)
+from .handlers.extension_manager_handler import ExtensionHandler, extensions_handler_path
 
 DEV_NOTE = """You're running JupyterLab from source.
 If you're working on the TypeScript sources of JupyterLab, try running
@@ -84,7 +81,10 @@ build_aliases["debug-log-path"] = "DebugLogFileMixin.debug_log_path"
 
 build_flags = dict(base_flags)
 
-build_flags["dev-build"] = ({"LabBuildApp": {"dev_build": True}}, "Build in development mode.")
+build_flags["dev-build"] = (
+    {"LabBuildApp": {"dev_build": True}},
+    "Build in development mode.",
+)
 build_flags["no-minimize"] = (
     {"LabBuildApp": {"minimize": False}},
     "Do not minimize a production build.",
@@ -98,9 +98,9 @@ build_flags["splice-source"] = (
 version = __version__
 app_version = get_app_version()
 if version != app_version:
-    version = "%s (dev), %s (app)" % (__version__, app_version)
+    version = f"{__version__} (dev), {app_version} (app)"
 
-buildFailureMsg = """Build failed.
+build_failure_msg = """Build failed.
 Troubleshooting: If the build failed due to an out-of-memory error, you
 may be able to fix it by disabling the `dev_build` and/or `minimize` options.
 
@@ -167,7 +167,9 @@ class LabBuildApp(JupyterApp, DebugLogFileMixin):
     )
 
     minimize = Bool(
-        True, config=True, help="Whether to minimize a production build (defaults to True)."
+        True,
+        config=True,
+        help="Whether to minimize a production build (defaults to True).",
     )
 
     pre_clean = Bool(
@@ -200,7 +202,7 @@ class LabBuildApp(JupyterApp, DebugLogFileMixin):
                     minimize=self.minimize,
                 )
             except Exception as e:
-                print(buildFailureMsg)
+                self.log.error(build_failure_msg)
                 raise e
 
 
@@ -214,8 +216,14 @@ clean_flags["extensions"] = (
     {"LabCleanApp": {"extensions": True}},
     "Also delete <app-dir>/extensions.\n%s" % ext_warn_msg,
 )
-clean_flags["settings"] = ({"LabCleanApp": {"settings": True}}, "Also delete <app-dir>/settings")
-clean_flags["static"] = ({"LabCleanApp": {"static": True}}, "Also delete <app-dir>/static")
+clean_flags["settings"] = (
+    {"LabCleanApp": {"settings": True}},
+    "Also delete <app-dir>/settings",
+)
+clean_flags["static"] = (
+    {"LabCleanApp": {"static": True}},
+    "Also delete <app-dir>/static",
+)
 clean_flags["all"] = (
     {"LabCleanApp": {"all": True}},
     "Delete the entire contents of the app directory.\n%s" % ext_warn_msg,
@@ -327,7 +335,7 @@ class LabWorkspaceApp(JupyterApp):
     There are three sub-commands for export, import or listing of workspaces. This app
         should not otherwise do any work.
     """
-    subcommands = dict()
+    subcommands = {}
     subcommands["export"] = (
         LabWorkspaceExportApp,
         LabWorkspaceExportApp.description.splitlines()[0],
@@ -344,7 +352,7 @@ class LabWorkspaceApp(JupyterApp):
     def start(self):
         try:
             super().start()
-            print("One of `export`, `import` or `list` must be specified.")
+            self.log.error("One of `export`, `import` or `list` must be specified.")
             self.exit(1)
         except NoStart:
             pass
@@ -404,7 +412,7 @@ aliases.update(
 )
 
 
-class LabApp(NBClassicConfigShimMixin, LabServerApp):
+class LabApp(NotebookConfigShimMixin, LabServerApp):
     version = version
 
     name = "lab"
@@ -451,11 +459,18 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
     aliases["app-dir"] = "LabApp.app_dir"
 
     flags = flags
-    flags["core-mode"] = ({"LabApp": {"core_mode": True}}, "Start the app in core mode.")
+    flags["core-mode"] = (
+        {"LabApp": {"core_mode": True}},
+        "Start the app in core mode.",
+    )
     flags["dev-mode"] = (
         {"LabApp": {"dev_mode": True}},
         "Start the app in dev mode for running from source.",
     )
+    flags["skip-dev-build"] = (
+        {"LabApp": {"skip_dev_build": True}},
+        "Skip the initial install and JS build of the app in dev mode.",
+    )
     flags["watch"] = ({"LabApp": {"watch": True}}, "Start the app in watch mode.")
     flags["splice-source"] = (
         {"LabApp": {"splice_source": True}},
@@ -463,8 +478,7 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
     )
     flags["expose-app-in-browser"] = (
         {"LabApp": {"expose_app_in_browser": True}},
-        """Expose the global app instance to browser via window.jupyterapp.
-        It is also available via the deprecated window.jupyterlab name.""",
+        "Expose the global app instance to browser via window.jupyterapp.",
     )
     flags["extensions-in-dev-mode"] = (
         {"LabApp": {"extensions_in_dev_mode": True}},
@@ -472,24 +486,23 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
     )
     flags["collaborative"] = (
         {"LabApp": {"collaborative": True}},
-        "Whether to enable collaborative mode.",
-    )
-    flags["future-skip-styles-for-disabled"] = (
-        {"LabApp": {"future_skip_styles_for_disabled": True}},
-        """Whether to skip loading styles for disabled prebuilt extensions.
-        This will be the default behavior starting with JupyterLab 4.0
-        (and this flag will be removed).""",
+        """To enable real-time collaboration, you must install the extension `jupyter_collaboration`.
+        You can install it using pip for example:
+
+            python -m pip install jupyter_collaboration
+
+        This flag is now deprecated and will be removed in JupyterLab v5.""",
     )
 
-    subcommands = dict(
-        build=(LabBuildApp, LabBuildApp.description.splitlines()[0]),
-        clean=(LabCleanApp, LabCleanApp.description.splitlines()[0]),
-        path=(LabPathApp, LabPathApp.description.splitlines()[0]),
-        paths=(LabPathApp, LabPathApp.description.splitlines()[0]),
-        workspace=(LabWorkspaceApp, LabWorkspaceApp.description.splitlines()[0]),
-        workspaces=(LabWorkspaceApp, LabWorkspaceApp.description.splitlines()[0]),
-        licenses=(LabLicensesApp, LabLicensesApp.description.splitlines()[0]),
-    )
+    subcommands = {
+        "build": (LabBuildApp, LabBuildApp.description.splitlines()[0]),
+        "clean": (LabCleanApp, LabCleanApp.description.splitlines()[0]),
+        "path": (LabPathApp, LabPathApp.description.splitlines()[0]),
+        "paths": (LabPathApp, LabPathApp.description.splitlines()[0]),
+        "workspace": (LabWorkspaceApp, LabWorkspaceApp.description.splitlines()[0]),
+        "workspaces": (LabWorkspaceApp, LabWorkspaceApp.description.splitlines()[0]),
+        "licenses": (LabLicensesApp, LabLicensesApp.description.splitlines()[0]),
+    }
 
     default_url = Unicode("/lab", config=True, help="The default URL to redirect to from `/`")
 
@@ -498,7 +511,8 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
     )
 
     override_theme_url = Unicode(
-        config=True, help=("The override url for static lab theme assets, typically a CDN.")
+        config=True,
+        help=("The override url for static lab theme assets, typically a CDN."),
     )
 
     app_dir = Unicode(None, config=True, help="The app directory to launch JupyterLab from.")
@@ -540,25 +554,40 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
         against published packages may not work correctly.""",
     )
 
+    extension_manager = Unicode(
+        "pypi",
+        config=True,
+        help="""The extension manager factory to use. The default options are:
+        "readonly" for a manager without installation capability or "pypi" for
+        a manager using PyPi.org and pip to install extensions.""",
+    )
+
     watch = Bool(False, config=True, help="Whether to serve the app in watch mode")
 
+    skip_dev_build = Bool(
+        False,
+        config=True,
+        help="Whether to skip the initial install and JS build of the app in dev mode",
+    )
+
     splice_source = Bool(False, config=True, help="Splice source packages into app directory.")
 
     expose_app_in_browser = Bool(
         False,
         config=True,
-        help="Whether to expose the global app instance to browser via window.jupyterlab",
+        help="Whether to expose the global app instance to browser via window.jupyterapp",
     )
 
-    future_skip_styles_for_disabled = Bool(
+    collaborative = Bool(
         False,
         config=True,
-        help="""Whether to skip loading styles for disabled prebuilt extensions.
-        This will be the default behavior starting with JupyterLab 4.0
-        (and this flag will be removed).""",
-    )
+        help="""To enable real-time collaboration, you must install the extension `jupyter_collaboration`.
+        You can install it using pip for example:
 
-    collaborative = Bool(False, config=True, help="Whether to enable collaborative mode.")
+            python -m pip install jupyter_collaboration
+
+        This flag is now deprecated and will be removed in JupyterLab v5.""",
+    )
 
     news_url = Unicode(
         "https://jupyterlab.github.io/assets/feed.xml",
@@ -618,7 +647,7 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
         if self.override_static_url:
             return self.override_static_url
         else:
-            static_url = "/static/{name}/".format(name=self.name)
+            static_url = f"/static/{self.name}/"
             return ujoin(self.serverapp.base_url, static_url)
 
     @default("theme_url")
@@ -638,11 +667,11 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
             self.log.info("Running JupyterLab in dev mode")
 
         if self.watch and self.core_mode:
-            self.log.warn("Cannot watch in core mode, did you mean --dev-mode?")
+            self.log.warning("Cannot watch in core mode, did you mean --dev-mode?")
             self.watch = False
 
         if self.core_mode and self.dev_mode:
-            self.log.warn("Conflicting modes, choosing dev_mode over core_mode")
+            self.log.warning("Conflicting modes, choosing dev_mode over core_mode")
             self.core_mode = False
 
         # Set the paths based on JupyterLab's mode.
@@ -651,8 +680,18 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
             self.static_paths = [dev_static_dir]
             self.template_paths = [dev_static_dir]
             if not self.extensions_in_dev_mode:
-                self.labextensions_path = []
-                self.extra_labextensions_path = []
+                # Add an exception for @jupyterlab/galata-extension
+                galata_extension = pjoin(HERE, "galata")
+                self.labextensions_path = (
+                    [galata_extension]
+                    if galata_extension in map(os.path.abspath, self.labextensions_path)
+                    else []
+                )
+                self.extra_labextensions_path = (
+                    [galata_extension]
+                    if galata_extension in map(os.path.abspath, self.extra_labextensions_path)
+                    else []
+                )
         elif self.core_mode:
             dev_static_dir = ujoin(HERE, "static")
             self.static_paths = [dev_static_dir]
@@ -663,11 +702,7 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
             self.static_paths = [self.static_dir]
             self.template_paths = [self.templates_dir]
 
-    def initialize_settings(self):
-        super().initialize_settings()
-
-    def initialize_handlers(self):
-
+    def initialize_handlers(self):  # noqa
         handlers = []
 
         # Set config for Jupyterlab
@@ -678,9 +713,7 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
         page_config["token"] = self.serverapp.token
         page_config["exposeAppInBrowser"] = self.expose_app_in_browser
         page_config["quitButton"] = self.serverapp.quit_button
-        page_config["collaborative"] = self.collaborative
         page_config["allow_hidden_files"] = self.serverapp.contents_manager.allow_hidden
-        page_config["futureSkipStylesForDisabled"] = self.future_skip_styles_for_disabled
 
         # Client-side code assumes notebookVersion is a JSON-encoded string
         page_config["notebookVersion"] = json.dumps(jpserver_version_info)
@@ -704,7 +737,7 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
             self.log.info(CORE_NOTE.strip())
             ensure_core(self.log)
         elif self.dev_mode:
-            if not self.watch:
+            if not (self.watch or self.skip_dev_build):
                 ensure_dev(self.log)
                 self.log.info(DEV_NOTE)
         else:
@@ -727,8 +760,60 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
             self.cache_files = False
 
         if not self.core_mode and not errored:
-            ext_manager = ExtensionManager(app_options=build_handler_options)
-            ext_handler = (extensions_handler_path, ExtensionHandler, {"manager": ext_manager})
+            # Add extension management handlers
+            provider = self.extension_manager
+            entry_point = EXT_MANAGERS.get(provider)
+            if entry_point is None:
+                self.log.error(f"Extension Manager: No manager defined for provider '{provider}'.")
+                raise NotImplementedError()
+            else:
+                self.log.info(f"Extension Manager is '{provider}'.")
+            manager_factory = entry_point.load()
+            config = self.settings.get("config", {}).get("LabServerApp", {})
+
+            blocked_extensions_uris = config.get("blocked_extensions_uris", "")
+            allowed_extensions_uris = config.get("allowed_extensions_uris", "")
+
+            if (blocked_extensions_uris) and (allowed_extensions_uris):
+                self.log.error(
+                    "Simultaneous LabServerApp.blocked_extensions_uris and LabServerApp.allowed_extensions_uris is not supported. Please define only one of those."
+                )
+                import sys
+
+                sys.exit(-1)
+
+            listings_config = {
+                "blocked_extensions_uris": set(
+                    filter(lambda uri: len(uri) > 0, blocked_extensions_uris.split(","))
+                ),
+                "allowed_extensions_uris": set(
+                    filter(lambda uri: len(uri) > 0, allowed_extensions_uris.split(","))
+                ),
+                "listings_refresh_seconds": config.get("listings_refresh_seconds", 60 * 60),
+                "listings_tornado_options": config.get("listings_tornado_options", {}),
+            }
+            if len(listings_config["blocked_extensions_uris"]) or len(
+                listings_config["allowed_extensions_uris"]
+            ):
+                self.log.debug(f"Extension manager will be constrained by {listings_config}")
+
+            try:
+                ext_manager = manager_factory(build_handler_options, listings_config, self)
+                metadata = dataclasses.asdict(ext_manager.metadata)
+            except Exception as err:
+                self.log.warning(
+                    f"Failed to instantiate the extension manager {provider}. Falling back to read-only manager.",
+                    exc_info=err,
+                )
+                ext_manager = ReadOnlyExtensionManager(build_handler_options, listings_config, self)
+                metadata = dataclasses.asdict(ext_manager.metadata)
+
+            page_config["extensionManager"] = metadata
+            ext_handler = (
+                extensions_handler_path,
+                ExtensionHandler,
+                {"manager": ext_manager},
+            )
             handlers.append(ext_handler)
 
             # Add announcement handlers
@@ -779,16 +864,21 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
     def initialize(self, argv=None):
         """Subclass because the ExtensionApp.initialize() method does not take arguments"""
         super().initialize()
-
-        if self.collaborative and jpserver_version_info < (2, 0, 0):
-            jpserver_version = ".".join(filter(lambda p: len(p) > 0, map(str, jpserver_version_info)))
-            self.log.critical(f"""To enable real-time collaboration, you must install Jupyter Server v2 (current version is {jpserver_version}).
-
+        if self.collaborative:
+            try:
+                import jupyter_collaboration  # noqa
+            except ImportError:
+                self.log.critical(
+                    """
+To enable real-time collaboration, you must install the extension `jupyter_collaboration`.
 You can install it using pip for example:
 
-  python -m pip install "jupyter_server>=2.0.0"
-""")
-            sys.exit(1)
+  python -m pip install jupyter_collaboration
+
+This flag is now deprecated and will be removed in JupyterLab v5.
+"""
+                )
+                sys.exit(1)
 
 
 # -----------------------------------------------------------------------------
